// build with:     cc -o serialports -framework CoreFoundation -framework IOKit serialports.c

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <paths.h>
#include <termios.h>
#include <sysexits.h>
#include <sys/param.h>
#include <sys/select.h>
#include <sys/time.h>
#include <time.h>
#include <AvailabilityMacros.h>

#ifdef __MWERKS__
#define __CF_USE_FRAMEWORK_INCLUDES__
#endif

#include <CoreFoundation/CoreFoundation.h>

#include <IOKit/IOKitLib.h>
#include <IOKit/serial/IOSerialKeys.h>
#if defined(MAC_OS_X_VERSION_10_3) && (MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_3)
#include <IOKit/serial/ioss.h>
#endif
#include <IOKit/IOBSD.h>

kern_return_t FindSerialPorts(io_iterator_t *matchingServices)
{
    kern_return_t			kernResult; 
    CFMutableDictionaryRef	classesToMatch;

    // Serial devices are instances of class IOSerialBSDClient
    classesToMatch = IOServiceMatching(kIOSerialBSDServiceValue);
    if (classesToMatch == NULL)
    {
        printf("IOServiceMatching returned a NULL dictionary.\n");
    }
    else {
        CFDictionarySetValue(classesToMatch,
                             CFSTR(kIOSerialBSDTypeKey),
                             CFSTR(kIOSerialBSDAllTypes));
    }
    kernResult = IOServiceGetMatchingServices(kIOMasterPortDefault, classesToMatch, matchingServices);    
    if (KERN_SUCCESS != kernResult)
    {
        printf("IOServiceGetMatchingServices returned %d\n", kernResult);
		goto exit;
    }
        
exit:
    return kernResult;
}

int GetSerialPaths()
{
    io_iterator_t       theSerialIterator;
    io_object_t		theObject;
    kern_return_t	kernResult = KERN_FAILURE;
    char bsdPath[256];
    int count = 0;

    if (FindSerialPorts(&theSerialIterator) != KERN_SUCCESS)
    {
        printf("createSerialIterator failed\n");
        return 0;
    }
    
    while (theObject = IOIteratorNext(theSerialIterator))
    {
        CFTypeRef	bsdPathAsCFString;

	bsdPathAsCFString = IORegistryEntryCreateCFProperty(theObject,
                                                            CFSTR(kIOCalloutDeviceKey),
                                                            kCFAllocatorDefault,
                                                            0);
        if (bsdPathAsCFString)
        {
            Boolean result;
            
	    result = CFStringGetCString(bsdPathAsCFString,
                                        bsdPath,
                                        sizeof(bsdPath), 
                                        kCFStringEncodingUTF8);
            CFRelease(bsdPathAsCFString);
            
            if (result)
	    {
	        printf("Serial port %d: path: %s", count, bsdPath);
            }
        }

        printf("\n");

        // Release the io_service_t now that we are done with it.
	
	(void) IOObjectRelease(theObject);
	count++;
    }
        
    return 1;
}

main()
{
  GetSerialPaths();
}  
